(define-module (ui portsctrl ajax)
               :use-module (alterator ajax)
               :use-module (alterator woo)
               :use-module (alterator card-index)
               :export (init))

; Debug string (paste in for your needs).
;(format #t "XXX: ~S\n" rdp_profile_name)


; Select some USB rule
(define (usb-rule-selected)
      (form-update-activity "btn_usb_rule_remove" #t))

; Select present USB device
(define (usb-prsnt-dev-selected)
      (form-update-activity "btn_prsnt_dev_add" #t))

; Update widgets status
(define (widgets-update)

  (format #t "[ajax,portsctrl] Update widgets status\n")

  (form-update-activity "btn_serial_save" #f)
  (form-update-activity "btn_usb_rule_remove" #f)
  (form-update-activity "btn_prsnt_dev_add" #f)
  (form-update-value "label_serial_selected" (_ "please choose from the list above" "alterator-ports-access")))


; Reset all values
(define (widgets-reset)

  (format #t "[ajax,portsctrl] Reset all widgets\n")

  (for-each (lambda(p) (form-update-value p ""))
            '("label_usb_control"
              "label_serial_control"
              "label_serial_selected"
              "list_serial_enabled"
              "input_usb_vendor"
              "input_usb_productid"
              "input_usb_info"
              "input_usb_serial")))


; Allow USB hid
(define (allow-usb-hid)

  (catch/message
    (lambda()
      (woo-write "/portsctrl/usb_hid"
                 'allow_hid (form-value "bool_allow_hid")))))


; Force scan USB device
(define (prsnt-scan)
  (format #t "[ajax,portsctrl] Force scan present devices\n")

  (form-update-enum "list_prsnt_devices"
                    (woo-list "/portsctrl/list_prsnt_devices"
                              'force "yes"
                              'language (form-value "language")))

  (form-update-activity "btn_prsnt_dev_add" #f)

  (form-bind "list_prsnt_devices" "change" usb-prsnt-dev-selected))


; Read current settings
(define (read-current-status)
  (format #t "[ajax,portsctrl] Read current status\n")

  (and

    ; Reset widgets
    (widgets-reset)

    ; Fill lists
    (for-each
      (lambda(lst)
        (form-update-enum lst
                          (woo-list
                            (string-append "/portsctrl/" lst)
                            'language (form-value "language"))))
      '("list_serial" "list_usb_rules" "list_serial_enabled" "list_prsnt_devices"))

    ; Rebind actions on lists
    (form-bind "list_serial" "change" serial-edit)
    (form-bind "list_usb_rules" "change" usb-rule-selected)
    (form-bind "list_prsnt_devices" "change" usb-prsnt-dev-selected)

    ; Read current settings
    (let* ((settings
             (woo-read-first "/portsctrl/status"
                             'language (form-value "language")))
           (usb-ctrl-on (woo-get-option settings 'usb_ctrl_on #f))
           (usb-hid (woo-get-option settings 'usb_hid #f))
           (serial-ctrl-on (woo-get-option settings 'serial_ctrl_on #f)))

      (form-update-value "label_serial_control" (if serial-ctrl-on (_ "Serial ports control activated" "alterator-ports-access") (_ "Serial ports control deactivated" "alterator-ports-access")))
      (form-update-value "label_usb_control" (if usb-ctrl-on (_ "USB ports control activated" "alterator-ports-access") (_ "USB ports control deactivated" "alterator-ports-access")))
      (form-update-value "bool_allow_hid" usb-hid)
      (form-update-visibility "btn_serial_off" serial-ctrl-on)
      (form-update-visibility "btn_serial_on" (not serial-ctrl-on))
      (form-update-visibility "btn_usb_off" usb-ctrl-on)
      (form-update-visibility "btn_usb_on" (not usb-ctrl-on)))

    ; Update widgets status
    (widgets-update)))


; Change control
(define (control proto enabled)

  (format #t "[ajax,portsctrl] change control for ~S to ~S\n" proto enabled)

  (catch/message
    (lambda()
      (woo-write "/portsctrl/ctrl"
                 'proto proto
                 'enabled enabled
                 'language (form-value "language"))))

  (read-current-status))


; Turn ON serial control
(define (serial-ctrl-on)
  (control "serial" #t))


; Turn OFF serial control
(define (serial-ctrl-off)
  (control "serial" #f))


; Turn ON USB control
(define (usb-ctrl-on)
  (control "usb" #t))


; Turn OFF USB control
(define (usb-ctrl-off)
  (control "usb" #f))


; Save serial port settings
(define (serial-save)

  (format #t "[ajax,portsctrl] Save serial port settings\n")

  (catch/message
    (lambda()
      (woo-write "/portsctrl/serial"
                 'port (form-value "list_serial")
                 'enabled (form-value "list_serial_enabled")
                 'language (form-value "language"))))

  (read-current-status))


; Remove selected entry from USB whitelist
(define (usb-rule-remove)

  (format #t "[ajax,portsctrl] Remove selected rule from whitelist\n")

  (catch/message
    (lambda()
      (woo-write "/portsctrl/usb_rm"
                 'rule_num (f-get-ci-value "list_usb_rules")
                 'language (form-value "language"))))

  (read-current-status))



; Add new USB rule to white list
(define (usb-rule-new)

  (format #t "[ajax,portsctrl] Add new entry to USB whitelist\n")

  (catch/message
    (lambda()
      (woo-write "/portsctrl/usb_add"
                 'vendor (form-value "input_usb_vendor")
                 'productid (form-value "input_usb_productid")
                 'serial (form-value "input_usb_serial")
                 'info (form-value "input_usb_info")
                 'language (form-value "language"))))

  (read-current-status))


; Add present USB device to white list
(define (prsnt-dev-add)

  (format #t "[ajax,portsctrl] Add present USB device to whitelist\n")

  (catch/message
    (lambda()
      (woo-write "/portsctrl/usb_add_prsnt"
                 'vendor_product (f-get-ci-value "list_prsnt_devices")
                 'language (form-value "language"))))

  (read-current-status))


; Edit serial port setting
(define (serial-edit)
  (format #t "[ajax,portsctrl] Edit serial port settings\n")

  (let* ((port (f-get-ci-value "list_serial")))
    (and
      (string? port)

      ; Read current port settings
      (let* ((settings
               (woo-read-first "/portsctrl/serial_port"
                               'port port
                               'language (form-value "language"))))
        (form-update-value-list '("label_serial_selected" "list_serial_enabled") settings))

      (form-update-activity "btn_serial_save" #t))))


; Initialization routine
(define (init)

  (format #t "[ajax,portsctrl] Run initialization routine\n")

  ; Interconnect button with action
  (form-bind "btn_serial_save" "click" serial-save)
  (form-bind "btn_usb_rule_remove" "click" usb-rule-remove)
  (form-bind "btn_usb_new_rule" "click" usb-rule-new)
  (form-bind "btn_prsnt_dev_add" "click" prsnt-dev-add)
  (form-bind "btn_prsnt_scan" "click" prsnt-scan)
  (form-bind "bool_allow_hid" "change" allow-usb-hid)

  (form-bind "btn_serial_on" "click" serial-ctrl-on)
  (form-bind "btn_serial_off" "click" serial-ctrl-off)
  (form-bind "btn_usb_on" "click" usb-ctrl-on)
  (form-bind "btn_usb_off" "click" usb-ctrl-off)

  ; Interface initialization
  (read-current-status))

; vim: ft=lisp expandtab
